#define __PROG_TYPES_COMPAT__

#define F_CPU			16000000

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>

#include "lcd.h"
#include "gfx.h"
#include "utils.h"

int frecventa_nota[32] = {
	131, 147, 165, 175, 196, 220, 247, // -1 ocatava fara bemoli
	262, 277, 295, 311, 330, 350, 372, 392, 415, 440, 466, 494, //octava principala
	523, 554, 587, 622, 659, 698, 740, 784, 831, 880, 932, 988, // +1 octava
	1047
};

// Variabile folosite pentru alegerea modului de joc.
static int modeSet = 0, filledMode = 1, dif, arrow = 0;

// Coordonatele capului rachetei.
static int  posX, posY;
// Matrice folosite pentru implementarea jocului (starea actuala si cea trecuta a jocului).
static char mat[22][22], oMat[22][22];

// Variabile folosite in intreruperi.
volatile int8_t stop = 0, start = 0, note = 0;

// Variabile folosite pentru memorarea starii jocului.
int8_t lvl = 1, destruction = 0, died = 0, lives = 3, level = 1;

// Vectori utili pentru reducerea dimensiunii codului.
uint8_t dx[4] = {-1, 0, 1, 0}, dy[4] = {0, -1, 0, 1};
int dificulty1[3] = { 0, 10, 10 }, dificulty2[3] = { 0, 12, 13 }, dificulty3[3] = { 0, 3, 4 };
int nr_steps[13] = { 0, 50, 50, 60, 60, 70, 70, 80, 80, 100, 100, 120, 120 };
int dix[4][7], diy[4][7], adx[4][4], ady[4][4];

// Variabila folosita la generatorul de numere random.
int seed = 777;

// Variabile folosite pentru a seta durata de redare a sunetelor.
int times = DEF_TIME, actSong = 0, special = 1, special2 = 1;

// Genereaza o nota folosind speaker-ul.
void speaker_reda(int nota) {
	int perioada_nota = 1000000l / frecventa_nota[nota];
	int i, j;

	/*
	* Nu putem folosi variable ca argument pentru functia
	* _delay_us(), asa ca folosim acest "workaround".
	*/
	if (actSong) {
		times = special * frecventa_nota[nota] / 4;
		times /= special2;
	}

	for (j = 0; j < times; ++j) {
		PORTA |= (1 << BUZZER);
		for (i = 0; i < perioada_nota / 2; ++i)
			_delay_us(1);

		PORTA &= ~(1 << BUZZER);
		for (i = 0; i < perioada_nota / 2; ++i)
			_delay_us(1);
	}

}

// Generator de numere random.
void myrand() {
	unsigned long long next;
	next = 1LL * (seed + 4772347) * 777013;
	seed = next % 37773;
	srand(seed);
}

// Functie care seteaza un buton.
static void button_set(uint8_t button)
{
	BUTTON_REG &= ~(1 << button);
	BUTTON_PORT |= (1 << button);
}

// Functie care scrie pe ecran "Welcome!"
static void write_welcome()
{
	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_GREEN);
	GFX_set_cursor(23, 58);
	GFX_print_str("Welcome!");
}

// Functie care scrie pe ecran "pagina" de start a jocului.
static void write_menu()
{
	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_BLUE);
	GFX_set_cursor(30, 20);
	GFX_print_str("Rocket");

	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_BLUE);
	GFX_set_cursor(42, 40);
	GFX_print_str("Game");

	GFX_set_text_size(1);
	GFX_set_text_color(ST7735_RED);

	GFX_set_cursor(36, 90);
	GFX_print_str("Press FIRE");

	GFX_set_cursor(43, 100);
	GFX_print_str("to start");
}

// Functie care afiseaza numarul de vieti ramase.
void write_lives(uint8_t nL) {
	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_RED);
	GFX_set_cursor(63, 50);
	GFX_print_char('0' + nL);

	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_RED);
	if (nL > 1) {
		GFX_set_cursor(7, 70);
		GFX_print_str("lives left");
	} else {
		GFX_set_cursor(10, 70);
		GFX_print_str("life left");
	}
	
}

// Functie care afiseaza nivelul la care s-a ajuns.
void write_level(uint8_t nL) {
	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_BLUE);
	GFX_set_cursor(40, 50);
	GFX_print_str("Level");

	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_BLUE);
	if (nL < 10) {
		GFX_set_cursor(64, 70);
		GFX_print_char('0' + nL);
	} else {
		GFX_set_cursor(58, 70);
		GFX_print_char('0' + (nL / 10));
		GFX_set_cursor(68, 70);
		GFX_print_char('0' + (nL % 10));
	}
}

// Functie care afiseaza "Game over!" pe ecran.
void write_gameover() {
	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_RED);
	GFX_set_cursor(9, 60);
	GFX_print_str("Game over!");
}

// Functie care deseneaza o sageata pentru meniul de alegere al modului de joc.
void drawArrow(uint8_t x, uint8_t y, int color) {
	uint8_t i;

	LCD_draw_pixel(x, y, color);
	LCD_draw_pixel(x - 1, y - 1, color);
	LCD_draw_pixel(x - 2, y - 2, color);
	LCD_draw_pixel(x - 3, y - 3, color);
	LCD_draw_pixel(x - 1, y + 1, color);
	LCD_draw_pixel(x - 2, y + 2, color);
	LCD_draw_pixel(x - 3, y + 3, color);
	
	for (i = 1; i <= 7; ++i)
		LCD_draw_pixel(x - i, y, color);
}

// Functie care afiseaza pe ecran un patrat DIM X DIM in functie de modul de joc ales.
void printSq(uint8_t x, uint8_t y, int color) {
	if (!filledMode) {
		if (color == ST7735_BLACK)
			GFX_draw_rect(ox + padx + (x - 1) * DIM, oy + pady + (y - 1) * DIM, DIM, DIM, color, 1);
		else {
			GFX_draw_rect(ox + padx + (x - 1) * DIM, oy + pady + (y - 1) * DIM, DIM - 1, DIM - 1, color, 0);
			GFX_draw_rect(ox + padx + 2 + (x - 1) * DIM, oy + pady + 2 + (y - 1) * DIM, DIM - 4, DIM - 4, color, 1);
			LCD_draw_pixel(ox + padx + x * DIM - 1, oy + pady + y * DIM - 1, color);
		}
	} else {
		if (color != ST7735_CYAN) {
			GFX_draw_rect(ox + padx + (x - 1) * DIM, oy + pady + (y - 1) * DIM, DIM, DIM, color, 1);
		} else {
			GFX_draw_rect(ox + padx + 1 + (x - 1) * DIM, oy + pady + 1 + (y - 1) * DIM, DIM - 2, DIM - 2, color, 1);
		}
	}
}

// Functie care deseneaza o racheta pozitionata cu capul in sus.
void drawRocket(uint8_t x, uint8_t y) {
	int co;

	lvl = 1;
	for (co = 0; co <= 6; ++co) {
		printSq(x + dix[lvl][co], y + diy[lvl][co], ST7735_MAGENTA);
	}
}

// Functie care umple ecranul cu culoarea data ca parametru.
void fill(uint8_t color) {
	LCD_fill_screen(color);
	GFX_draw_rect(128, 3, 2, 128, color, 1);
}

//  Handler-ul folosit pentru intreruperile pe butoanele: FIRE, UP, DOWN.
ISR(PCINT0_vect) {
	if (!(BUTTON_PIN & (1 << BUTTON_FIRE))) {
		start = 1;
		stop = 1;
		if (!note) {
			note = 1;
			speaker_reda(19);
		}

		if (!modeSet) {
			filledMode = arrow;
			modeSet = 1;
		}
	}

	if (!(BUTTON_PIN & (1 << BUTTON_UP))) {
		if (arrow == 0 && !modeSet) {
			arrow = 1;
			speaker_reda(19);
		}
	}

	if (!(BUTTON_PIN & (1 << BUTTON_DOWN))) {
		if (arrow == 1 && !modeSet) {
			arrow = 0;
			speaker_reda(19);
		}
	}
}

// Functie care ajuta la determinarea modului de joc.
void write_modeMenu() {
	GFX_set_text_size(1.5);
	GFX_set_text_color(ST7735_BLUE);
	GFX_set_cursor(29, 20);
	GFX_print_str("Playing mode");

	GFX_set_text_size(1);
	GFX_set_cursor(31, 55);
	GFX_set_text_color(ST7735_GREEN);
	GFX_print_str("Filled");

	GFX_set_cursor(31, 97);
	GFX_print_str("Partial");

	filledMode = 1;
	drawRocket(17, 8);
	filledMode = 0;
	drawRocket(17, 15);
	filledMode = 1;

	drawArrow(27, 58, ST7735_YELLOW);
	arrow = 1;
	start = 0;
	modeSet = 0;
	while (!start) {
		if (arrow == 1) {
			drawArrow(27, 100, ST7735_BLACK);
			drawArrow(27, 58, ST7735_YELLOW);
		} else {
			drawArrow(27, 58, ST7735_BLACK);
			drawArrow(27, 100, ST7735_YELLOW);
		}
	}
}

// Functie care afiseaza pe ecran mesajul corespunzator trecerii tuturor nivelelor.
void write_victory() {
	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_YELLOW);
	GFX_set_cursor(23, 45);
	GFX_print_str("You won!");

	GFX_set_text_size(1);
	GFX_set_text_color(ST7735_GREEN);
	GFX_set_cursor(21, 70);
	GFX_print_str("Congratulations!");
}

// Functie care afiseaza pe ecran mesaje legate de vieti extra primite pe parcurs.
void write_extraLife(int nL) {
	GFX_set_text_size(2);
	GFX_set_text_color(ST7735_GREEN);
	if (nL == 1) {
		GFX_set_cursor(25, 60);
		GFX_print_str("+1 life");
	} else {
		GFX_set_cursor(20, 60);
		GFX_print_str("+2 lives");
	}
}

// Functie care seteaza valorile unor matrici folosite pentru "construirea" rachetei.
void setDI() {
	dix[1][0] = 0; diy[1][0] = 0;
	dix[1][1] = 0; diy[1][1] = 1;
	dix[1][2] = 0; diy[1][2] = 2;
	dix[1][3] = -1; diy[1][3] = 1;
	dix[1][4] = 1; diy[1][4] = 1;
	dix[1][5] = -1; diy[1][5] = 3;
	dix[1][6] = 1; diy[1][6] = 3;

	dix[2][0] = 0; diy[2][0] = 0;
	dix[2][1] = -1; diy[2][1] = -1;
	dix[2][2] = -1; diy[2][2] = 0;
	dix[2][3] = -1; diy[2][3] = 1;
	dix[2][4] = -2; diy[2][4] = 0;
	dix[2][5] = -3; diy[2][5] = -1;
	dix[2][6] = -3; diy[2][6] = 1;

	dix[3][0] = 0; diy[3][0] = 0;
	dix[3][1] = -1; diy[3][1] = -1;
	dix[3][2] = 0; diy[3][2] = -1;
	dix[3][3] = 1; diy[3][3] = -1;
	dix[3][4] = 0; diy[3][4] = -2;
	dix[3][5] = -1; diy[3][5] = -3;
	dix[3][6] = 1; diy[3][6] = -3;

	dix[0][0] = 0; diy[0][0] = 0;
	dix[0][1] = 1; diy[0][1] = -1;
	dix[0][2] = 1; diy[0][2] = 0;
	dix[0][3] = 1; diy[0][3] = 1;
	dix[0][4] = 2; diy[0][4] = 0;
	dix[0][5] = 3; diy[0][5] = -1;
	dix[0][6] = 3; diy[0][6] = 1;
}

/*
 * Functie care "introduce" racheta in matricea starii curente a jocului.
 * Aceasta functie verifica si coliziunile rachetei cu obstacolele.
 */
void setRocket(int x, int y) {
	int co;

	for (co = 0; co <= 6; ++co) {
		if (mat[x + dix[lvl][co]][y + diy[lvl][co]] > ROCKET) {
			died = 1;
			mat[x + dix[lvl][co]][y + diy[lvl][co]] = CRASH;
		} else
			mat[x + dix[lvl][co]][y + diy[lvl][co]] = ROCKET;
	}
}

// Functe care sterge racheta din matricea starii curente a jocului.
void delRocket(uint8_t x, uint8_t y) {
	int co;

	for (co = 0; co <= 6; ++co) {
		mat[x + dix[lvl][co]][y + diy[lvl][co]] = 0;
	}
}

// Functie care verifica daca racheta cu varful in coordonatele date nu depaseste marginile ecranului.
int check(uint8_t x, uint8_t y) {
	switch (lvl)
	{
	case 1:
	case 3:
		if ((x >= 2) && (x <= 20))
			return 1;
		break;
	case 0:
	case 2:
		if ((y >= 2) && (y <= 20))
			return 1;
		break;
	}
	
	return 0;
}

// Functie care copiaza matricea de stare curenta in cea care retine starea anterioara.
void copyM() {
	uint8_t i, j;
	for (i = 1; i <= 21; ++i)
		for (j = 1; j <= 21; ++j)
			oMat[i][j] = mat[i][j];
}

// Functie care "goleste"/"reseteaza" matricea de stare curenta.
void eraseM() {
	uint8_t i, j;
	for (i = 1; i <= 21; ++i)
		for (j = 1; j <= 21; ++j)
			mat[i][j] = 0;
}

// Functie care afiseaza pe ecran starea curenta a jocului.
void printAll() {
	uint8_t i, j;

	for(i = 1; i <= 21; ++i)
		for (j = 1; j <= 21; ++j) {
			if (mat[i][j] != oMat[i][j]) {
				switch (mat[i][j])
				{
				case ROCKET:
					printSq(i, j, ST7735_MAGENTA);
					break;
				case WALL:
					printSq(i, j, ST7735_GREEN);
					break;
				case METEO:
					printSq(i, j, ST7735_BLUE);
					break;
				case BULLET:
					printSq(i, j, ST7735_CYAN);
					break;
				case CRASH:
					printSq(i, j, ST7735_RED);
					break;
				default:
					printSq(i, j, ST7735_BLACK);
					break;
				}
			}
			
		}
}

// Functie care reda melodia de start.
void sing_welcome() {
	int i;
	stop = 0;
	uint8_t v[114] = { 23, 23, 50, 23, 50, 19, 23, 50, 26, 50, 50, 14, 50, 50, //0-13
					19, 50, 14, 50, 11, 50, 16, 50, 18, 50, 17, 16, 50, 14, 23, 26, //->29
					28, 50, 24, 26, 50, 23, 50, 19, 21, 18, 50, //->40
					19, 50, 14, 50, 11, 50, 16, 50, 18, 50, 17, 16, 50, 14, 23, 26, //->56
					28, 50, 24, 26, 50, 23, 50, 19, 21, 18, 50, //->67
					26, 25, 24, 22, 23, 50, 15, 16, 19, 50, 16, 19, 21, 50, 26, 25, 24, 22, 50, 23, 50, 31, 50, 31, 31, //->92
					50, 50, 26, 25, 24, 22, 23, 50, 15, 16, 19, 50, 16, 19, 21, 50, 22, 50, 21, 50, 19 //->113
	};

	actSong = 1;
	special2 = 2;
	for (i = 0; i <= 112; ++i) {
		if (stop) {
			break;
		}
		if (v[i] == 50) {
			_delay_ms(125);
		} else {
			speaker_reda(v[i]);
		}
	}

	if (!stop) {
		special = 3;
		special2 = 4;
		speaker_reda(v[113]);
	}

	actSong = 0;
	special = 1;
	special2 = 1;
	stop = 0;
	times = DEF_TIME;
}

/*
 * Functie folosita pentru a muta zidurile, meteoritii si gloantele (eventual, a le scoate din scena)
 * conform directiei de inaintare a nivelului. 
 */
void advance() {
	int i, j;

	switch (lvl) {
	case 1:
		for (j = 2; j <= 21; ++j)
			for (i = 1; i <= 21; ++i) {
				switch (oMat[i][j - 1]) {
				case WALL:
				case METEO:
					mat[i][j] = oMat[i][j - 1];
					break;
				case BULLET:
					if (j > 2) {
						if (oMat[i][j - 2] == WALL || oMat[i][j - 2] == METEO) {
							destruction = 1;
							mat[i][j - 1] = 0;
						} else if (j > 3 && (oMat[i][j - 3] == WALL || oMat[i][j - 3] == METEO)) {
							destruction = 1;
							mat[i][j - 2] = 0;
						} else {
							mat[i][j - 2] = BULLET;
						}
					}
					break;
				default:
					break;
				}
			}
		break;
	case 0:
		for (i = 2; i <= 21; ++i)
			for (j = 1; j <= 21; ++j) {
				switch (oMat[i - 1][j]) {
				case WALL:
				case METEO:
					mat[i][j] = oMat[i - 1][j];
					break;
				case BULLET:
					if (i > 2) {
						if (oMat[i - 2][j] == WALL || oMat[i - 2][j] == METEO) {
							destruction = 1;
							mat[i - 1][j] = 0;
						} else if (i > 3 && (oMat[i - 3][j] == WALL || oMat[i - 3][j] == METEO)) {
							destruction = 1;
							mat[i - 2][j] = 0;
						} else {
							mat[i - 2][j] = BULLET;
						}
					}
					break;
				default:
					break;
				}
			}
		break;
	case 3:
		for (j = 20; j >= 1; --j)
			for (i = 1; i <= 21; ++i) {
				switch (oMat[i][j + 1]) {
				case WALL:
				case METEO:
					mat[i][j] = oMat[i][j + 1];
					break;
				case BULLET:
					if (j < 20) {
						if (oMat[i][j + 2] == WALL || oMat[i][j + 2] == METEO) {
							destruction = 1;
							mat[i][j + 1] = 0;
						} else if (j < 19 && (oMat[i][j + 3] == WALL || oMat[i][j + 3] == METEO)) {
							destruction = 1;
							mat[i][j + 2] = 0;
						} else {
							mat[i][j + 2] = BULLET;
						}
					}
					break;
				default:
					break;
				}
			}
		break;
	case 2:
		for (i = 20; i >= 1; --i)
			for (j = 1; j <= 21; ++j) {
				switch (oMat[i + 1][j]) {
				case WALL:
				case METEO:
					mat[i][j] = oMat[i + 1][j];
					break;
				case BULLET:
					if (i < 20) {
						if (oMat[i + 2][j] == WALL || oMat[i + 2][j] == METEO) {
							destruction = 1;
							mat[i + 1][j] = 0;
						} else if (i < 19 && (oMat[i + 3][j] == WALL || oMat[i + 3][j] == METEO)) {
							destruction = 1;
							mat[i + 2][j] = 0;
						} else {
							mat[i + 2][j] = BULLET;
						}
					}
					break;
				default:
					break;
				}
			}
		break;
	}
}

// Functie folosita pentru a adauga o noua linie cu obstacole (dupa ce "avanseaza" cele deja afisate).
void addLine() {
	myrand();

	int nr1 = rand() % dificulty1[dif];
	int nr2 = rand() % (dificulty2[dif] - nr1);
	int nrm, i;
	int diff = 21 - nr1 - nr2;
	
	int pos = 0;
	switch (lvl) {
	case 0:
	case 1:
		pos = 1;
		break;
	case 2:
	case 3:
		pos = 21;
		break;
	}

	switch (lvl) {
	case 1:
	case 3:
		for (i = 1; i <= nr1; ++i)
			mat[i][pos] = WALL;
		for (i = 22 - nr2; i <= 21; ++i)
			mat[i][pos] = WALL;
		break;
	case 0:
	case 2:
		for (i = 1; i <= nr1; ++i)
			mat[pos][i] = WALL;
		for (i = 22 - nr2; i <= 21; ++i)
			mat[pos][i] = WALL;
		break;
	}

	nrm = rand() % dificulty3[dif];
	switch (lvl) {
	case 1:
	case 3:
		for (i = 1; i <= nrm; ++i) {
			mat[nr1 + 1 + rand() % diff][pos] = METEO;
		}
		break;
	case 0:
	case 2:
		for (i = 1; i <= nrm; ++i) {
			mat[pos][nr1 + 1 + rand() % diff] = METEO;
		}
		break;
	}
}

// Functie care reda melodia de game over.
void sing_gameover() {
	actSong = 1;
	speaker_reda(14);
	speaker_reda(11);
	speaker_reda(7);
	special = 2;
	speaker_reda(9);
	special = 1;
	actSong = 0;
	times = DEF_TIME;
}

// Functie care reda melodia de terminare cu succes a jocului.
void sing_victory() {
	uint8_t i;
	stop = 0;
	uint8_t v[28] = { 23, 19, 23, 19, 26, 26, 26,
		23, 19, 23, 19, 21, 14, 14,
		23, 19, 23, 19, 26, 26, 26,
		23, 19, 21, 14, 19, 19, 19 };

	actSong = 1;
	for (i = 0; i <= 27; ++i) {
		if (stop) {
			break;
		}

		if (i == 6 || i == 13 || i == 20 || i == 27) {
			special = 2;
			speaker_reda(v[i]);
			special = 1;
		} else {
			speaker_reda(v[i]);
		}
	}

	stop = 0;
	special = 1;
	actSong = 0;
	times = DEF_TIME;
}

/*
 * Functie care seteaza coordonatele initiale ale capului rachetei in functie de sensul si directia de
 * inaintare specifica nivelului.
 */
void setRocketInitPos() {
	switch (lvl) {
	case 1:
		posX = 11;
		posY = 18;
		return;
	case 2:
		posX = 4;
		posY = 11;
		return;
	case 3:
		posX = 11;
		posY = 4;
		return;
	case 0:
		posX = 18;
		posY = 11;
		return;
	}
}

// Functie folosita pentru a misca racheta la apasarea butoanelor specifice.
void moveRocket() {
	switch (lvl) {
	case 1:
	case 3:
		if (!(BUTTON_PIN & (1 << BUTTON_LEFT))) {
			if (check(posX - 1, posY)) {
				speaker_reda(7);
				--posX;
			}
		} else if (!(BUTTON_PIN & (1 << BUTTON_RIGHT))) {
			if (check(posX + 1, posY)) {
				speaker_reda(7);
				++posX;
			}
		}
		break;
	case 0:
	case 2:
		if (!(BUTTON_PIN & (1 << BUTTON_UP))) {
			if (check(posX, posY - 1)) {
				speaker_reda(7);
				--posY;
			}
		} else if (!(BUTTON_PIN & (1 << BUTTON_DOWN))) {
			if (check(posX, posY + 1)) {
				speaker_reda(7);
				++posY;
			}
		}
		break;
	}
}

void sing_levelPassed() {
	uint8_t i;
	stop = 0;

	actSong = 1;
	uint8_t ve[27] = {
		7, 9, 7, 9, 11, 12, 14, 14, 14, 12, 11, 12, 14, 50, // 0-13
		7, 9, 7, 9, 11, 12, 14, 14, 14, 12, 11, 9, 7 // 14-26
	};

	special2 = 2;
	for (i = 0; i <= 26; ++i) {
		if (stop) {
			break;
		}

		if (ve[i] == 50) {
			_delay_ms(200);
		} else if (i == 12 || i == 26) {
			special = 2;
			speaker_reda(ve[i]);
			special = 1;
		} else {
			speaker_reda(ve[i]);
		}
	}

	actSong = 0;
	special2 = 1;
	stop = 0;
	times = DEF_TIME;
}

int main(void)
{
	uint8_t k;
	uint8_t nx, ny, step;

	// Setez butoanele.
	button_set(BUTTON_UP);
	button_set(BUTTON_DOWN);
	button_set(BUTTON_LEFT);
	button_set(BUTTON_RIGHT);
	button_set(BUTTON_FIRE);

	// Setez pinul BUZZER ca iesire.
	DDRA |= (1 << BUZZER);

	// Opresc buzzer-ul.
	PORTA &= ~(1 << BUZZER);

	// Activez PIN CHANGE INTERRUPT ENABLE pentru PCINT7:0
	PCICR |= (1 << PCIE0);
	// Activez intreruperea pe butoanele PA0-5
	PCMSK0 |= (1 << PCINT0) | (1 << PCINT1) | (1 << PCINT2) | (1 << PCINT3) | (1 << PCINT4);

	// Initializez matricele de stare.
	eraseM();
	copyM();

	// Initializez anumite matrici folosite ulterior.
	setDI();

	// Initializez LCD-ul.
	LCD_init();
	GFX_init(130, 131);
	fill(ST7735_BLACK);

	// Rulez partea introductiva.
	sei();
	write_welcome();
	_delay_ms(500);
	sing_welcome();
	cli();

	times = DEF_TIME;
	while (1) {
		// Afisez meniul principal al jocului.
		fill(ST7735_BLACK);
		write_menu();

		sei();
		start = 0;
		note = 0;
		while (start == 0);
		_delay_ms(300);

		// Rulez partea de alegerea a modului de joc.
		fill(ST7735_BLACK);
		write_modeMenu();
		cli();

		// Setez cateva variabile de stare.
		lives = 3;
		died = 0;
		dif = 1;

		// Rulez fiecare nivel.
		for (level = 1; level <= NUM_LEVELS; ++level) {
			// Cand jucatorul ajunge la nivelul 5, primeste o viata extra.
			if (level == 5  && !died) {
				fill(ST7735_BLACK);
				write_extraLife(1);
				++lives;
				_delay_ms(750);
			}

			// Cand jucatorul ajunge la nivelul 9, primeste 2 vieti extra.
			if (level == 9 && !died) {
				fill(ST7735_BLACK);
				dif = 2;
				write_extraLife(2);
				lives += 2;
				_delay_ms(750);
			}

			if (level <= 8) {
				// Primele 8 nivele au directie de parcurs bine definita.
				lvl = level % 4;
			} else {
				// Restul nivelelor au o directie de parcurs random.
				if (!died)
					lvl = rand() % 4;
			}

			// Afisez nivelul, doar daca nu am mai facut-o anterior.
			if (!died) {
				fill(ST7735_BLACK);
				write_level(level);
				_delay_ms(2000);
			}

			// Setez cateva variabile de stare.
			died = 0;
			eraseM();
			copyM();
			fill(ST7735_BLACK);
			setRocketInitPos();
			setRocket(posX, posY);

			// Afisez racheta.
			printAll();

			// Rulez toti pasii dintr-un nivel.
			for (step = 1; step <= nr_steps[level]; ++step) {
				destruction = 0;
				copyM();
				eraseM();

				// Rulez functia de "avansare" a obiectelor din jurul rachetei.
				advance();

				// Adaug noi linii cu obstacole doar pana la atingerea unui prag.
				if (step + 21 <= nr_steps[level])
					addLine();

				// Adaug un nou glont in scena, daca este cazul.
				if (!(BUTTON_PIN & (1 << BUTTON_FIRE))) {
					nx = posX + dx[lvl];
					ny = posY + dy[lvl];
					if (mat[nx][ny] != 0) {
						mat[nx][ny] = 0;
					} else {
						mat[nx][ny] = BULLET;
					}
					speaker_reda(19);
				}

				// Mut racheta (daca este cazul) si o introduc in matricea de stare.
				moveRocket();
				setRocket(posX, posY);

				// Afisez starea curenta a jocului.
				printAll();
				// Verific daca jucatorul s-a lovit de un obstacol.
				if (died) {
					// Redau sunetul de coliziune.
					for (k = 1; k <= 10; ++k) {
						times = 10;
						speaker_reda(16);
						times = DEF_TIME;
					}

					// Scad numarul de vieti.
					--lives;

					// Daca mai am vieti, afisez numarul lor.
					if (lives > 0) {
						fill(ST7735_BLACK);
						write_lives(lives);
						_delay_ms(2000);
						--level;
						break;
					} else {
						// Daca nu mai am vieti afisez mesajul de game over
						// si redau melodia corespunzator.
						level = 100;
						fill(ST7735_BLACK);
						write_gameover();
						sing_gameover();
						_delay_ms(3000);
						break;
					}
				} else if (destruction) {
					// In cazul in care un glont a lovit un perete sau un meteorit
					// redau sunetul corespunzator.
					speaker_reda(21);
				}
				_delay_ms(50);
			}

			if (level <= NUM_LEVELS && level != 100 && !died) {
				sei();
				sing_levelPassed();
				cli();
			}
		}

		// In cazul in care au fost terminate toate nivelele cu succes
		// afisez mesajul de victorie si rulez melodia corespunzatoare.
		if (level == NUM_LEVELS + 1) {
			fill(ST7735_BLACK);
			write_victory();
			sing_victory();
			_delay_ms(2000);
		}
	}

	return 0;
}
